#!/usr/bin/env python2.7
#coding:utf-8

import hjson
import os
import jinja2
import codecs
#import imp
import importlib

class JinjaGenFiles:

  def __init__ (self):
    pass

  def generate (self, templatePath, dataPath=None, outputPath=None):
    cwd = os.getcwd()
    #print(str(cwd))

    checkDataStr = True if type(dataPath) == str else False

    dataClass = None
    dataListClass = False
    itemClass = None
    if checkDataStr and dataPath.startswith("class://"):
      #print("ES CLASS")
      dataPath = dataPath[8:]
      #print("dataPath = " + dataPath)
      dataIdx = dataPath.find('?')
      if dataIdx > -1:
        dataArgs = dataPath[(dataIdx + 1):]
        #print("dataArgs = " + dataArgs)
        arrayDataArgs = dataArgs.split('&')
        for itemArrayDataArgs in arrayDataArgs:
          valuesItemDataArg = itemArrayDataArgs.split('=')
          if valuesItemDataArg[0] == 'type':
            dataClass = valuesItemDataArg[1]
          if valuesItemDataArg[0] == 'list' and valuesItemDataArg[1] == 'true':
            dataListClass = True
        dataPath = dataPath[0:dataIdx]
    if dataClass is not None:
      classPkgIdx = dataClass.rfind(".")
      classPkg = dataClass[0:classPkgIdx]
      className = dataClass[(classPkgIdx+1):]

      #components = dataClass.split('.')
      '''
      mod = __import__(components[0]+"."+components[1])
      print(str(dir(mod)))
      for comp in components[2:]:
        print(str(comp))
        mod = getattr(mod, comp)
        print(str(dir(mod)))
      '''
      #mod = __import__(classPkg)
      mod = importlib.import_module(classPkg)
      #print(str(dir(mod)))
      #print("classPkg5 = " + str(classPkg))
      #print("className5 = " + str(className))
      itemClass = getattr(mod, className)
      #print(str(itemClass))
      '''
      try:
        fp, pathname, description = imp.find_module(classPkg)
        itemClass = imp.load_module("%s" % (dataClass), fp, pathname, description)
        print itemClass
      except Exception as e:
        print e
      '''

    if templatePath is not None and not os.path.isabs(templatePath):
      templatePath = os.path.join(cwd, templatePath)

    if checkDataStr and dataPath is not None and not os.path.isabs(dataPath):
      dataPath = os.path.join(cwd, dataPath)

    if outputPath is not None and not os.path.isabs(outputPath):
      outputPath = os.path.join(cwd, outputPath)

    baseDir = os.path.dirname(templatePath)
    templateName = os.path.basename(templatePath)

    data = None
    if checkDataStr and dataPath is not None:
      #print("load data path = " + dataPath)
      #data = dict(self.__loadObj__(dataPath))
      if itemClass is not None:
        if dataListClass:
          data = list()
          listData = self.__loadObj__(dataPath)
          for listDataItem in listData:
            dataItem = getattr(itemClass, 'load')(listDataItem)
            data.append(dataItem)
        else:
          data = getattr(itemClass, 'load')(dataPath)
      else:
        data = self.__loadObj__(dataPath)
      #print(str(data))
    else:
      data = dataPath

    file_loader = jinja2.FileSystemLoader(baseDir)
    env = jinja2.Environment(loader=file_loader, extensions=['jinja2.ext.do'])
    env.filters['myUpper'] = self.myUpper
    env.tests['verdadero'] = self.is_verdadero
    env.tests['falso'] = self.is_falso
    template = env.get_template(templateName)

    if outputPath is None:
      output = None
      if data is None:
        output = template.render()
      else:
        #print("render with data")
        output = template.render(data=data)
      print(output)
    else:
      stream = None
      if data is None:
        stream = template.stream()
      else:
        stream = template.stream(data=data)
      stream.dump(outputPath, encoding='utf-8')

  def __saveObj__ (self, path, obj):
    fp = codecs.open(path, mode='w', encoding='utf-8')
    hjson.dump(obj, fp)

  def __loadObj__ (self, path):
    result = None
    fp = codecs.open(path, mode='r', encoding='utf-8')
    result = hjson.load(fp)
    return result

  def myUpper(self, value, suffix='00'):
    suffix = suffix if suffix is not None else ''
    return value.upper() + suffix

  def is_verdadero(self, value):
    return True if value == 'True' else False

  def is_falso(self, value):
    return False if value == 'True' else True


def test():
  t = jinja2.Template("Hello {{ something }}!")
  print(t.render(something="World"))
  t = jinja2.Template("My favorite numbers: {% for n in range(1,10) %}{{n}} " "{% endfor %}")
  print(t.render())

def test2():
  file_loader = jinja2.FileSystemLoader('/media/jmramoss/ALMACEN/pypi/genfiles/genfiles')
  env = jinja2.Environment(loader=file_loader)
  template = env.get_template('hello.txt')
  output = template.render()
  print(output)

def test3():
  file_loader = jinja2.FileSystemLoader('/media/jmramoss/ALMACEN/pypi/genfiles/genfiles')
  env = jinja2.Environment(loader=file_loader)
  template = env.get_template('lamb.txt')
  output = template.render(name=u'Jesús')
  print(output)

def test4():
  person = {}
  person['name'] = u'Jesús'
  person['animal'] = u'murciélago'

  file_loader = jinja2.FileSystemLoader('/media/jmramoss/ALMACEN/pypi/genfiles/genfiles')
  env = jinja2.Environment(loader=file_loader)
  template = env.get_template('data.txt')
  output = template.render(data=person)
  print(output)

if True and __name__ == '__main__':
  tools = JinjaGenFiles()
  #tools.generate("hello.txt")
  #tools.generate("data.txt", dataPath="/media/jmramoss/ALMACEN/pypi/genfiles/genfiles/data.hjson")
  #tools.generate("data.txt", dataPath="/media/jmramoss/ALMACEN/pypi/genfiles/genfiles/data.hjson", outputPath="/media/jmramoss/ALMACEN/pypi/genfiles/genfiles/data.out")
  #tools.generate("data.txt", dataPath="data.hjson", outputPath="data3.out")
  tools.generate("templates/items.txt", dataPath="class://templates/items.hjson?type=genfiles.templates.Item&list=true", outputPath="templates/items.out")
  #test4()
  #print("HOLA2")



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name='genfiles',
    version='0.0.31',
    url='https://github.com/tuaplicacionpropia/genfiles',
    download_url='https://github.com/tuaplicacionpropia/genfiles/archive/master.zip',
    author=u'tuaplicacionpropia.com',
    author_email='tuaplicacionpropia@gmail.com',
    description='Python library for generate files with jinja and hjson.',
    long_description='Python library for generate files with jinja and hjson.',
    keywords='jinja, hjson, files, generate',
    classifiers=[
      'Development Status :: 4 - Beta',
      'License :: OSI Approved :: MIT License',
      'Programming Language :: Python',
      'Programming Language :: Python :: 2.7',
      'Intended Audience :: Developers',
      'Topic :: Multimedia :: Graphics',
    ],
    scripts=[
      'bin/gf_generate.cmd', 'bin/gf_generate',
      'bin/gf_help.cmd', 'bin/gf_help'
    ],
    packages=find_packages(exclude=['tests']),
    #package_data={},
    #package_data={'': ['license.txt']},
    package_data={'genfiles': ['templates/*.txt', 'templates/*.hjson']},
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    license='MIT',
    install_requires=[
        'hjson>=2.0.2',
    ],
)




#!/usr/bin/env python2.7
#coding:utf-8

import codecs
import hjson

class Item(object):

  def __init__(self):
    self.name = ""
    self.surnames = ""

  def fullName (self):
    return (self.name if self.name is not None else "") + " " + (self.surnames if self.surnames is not None else "")

  @staticmethod
  def load (data):
    result = Item()

    dataObj = {}
    if type(data) == 'str':
      fp = codecs.open(data, mode='r', encoding='utf-8')
      dataObj = hjson.load(fp)
    elif type(data) == 'dict' or isinstance(data, dict):
      dataObj = data

    result.name = dataObj['name'] if 'name' in dataObj else None
    result.surnames = dataObj['surnames'] if 'surnames' in dataObj else None

    return result



r"""Command-line tool to bslideshow

Usage::

    $ bs_footage /home/mivideo.mp4 foreground.mp4 /output/fout.mp4

"""
import sys
import os
import tempfile
import genfiles
import shutil

def main():
  args = []
  for i in range(2, len(sys.argv)):
    args.append(sys.argv[i])

  getattr(sys.modules[__name__], sys.argv[1])(args)

#args: templatePath, dataPath=None, outputPath=None
def generate (args):
  #print("executing genfiles " + str(args))
  gf = genfiles.JinjaGenFiles()
  gf.generate(*args)

#args: type
def help (args):
  #print("executing help " + str(args))
  _show_help(*args)

def printFile (filePath):
  with open(filePath, 'r') as fin:
    print fin.read()

def printTemplate (name):
  filePath = os.path.join(os.path.join(os.path.dirname(__file__), 'templates'), name)
  print("")
  print(">>> template " + name + " (start)")
  printFile(filePath)
  print(">>> template " + name + " (end)")
  print("")

def copyTemplate (name, outDir):
  srcPath = os.path.join(os.path.join(os.path.dirname(__file__), 'templates'), name)
  dstPath = os.path.join(outDir, name)
  shutil.copyfile(srcPath, dstPath)

def _show_help (type=None):
  if type == 'base':
    printTemplate('data.hjson')
    printTemplate('header.txt')
    printTemplate('macros.txt')
    printTemplate('base.txt')
  elif type == 'inheritance':
    printTemplate('data.hjson')
    printTemplate('header.txt')
    printTemplate('parent.txt')
    printTemplate('child.txt')
  elif type == 'class':
    printTemplate('items.hjson')
    printTemplate('item.py')
    printTemplate('items.txt')
  elif type == 'data':
    printTemplate('data.hjson')
  elif type == 'test':
    cwd = os.getcwd()
    tmpDir = tempfile.mkdtemp(suffix='_genfiles', prefix='test_', dir=cwd)
    copyTemplate('base.txt', tmpDir)
    copyTemplate('child.txt', tmpDir)
    copyTemplate('data.hjson', tmpDir)
    copyTemplate('header.txt', tmpDir)
    copyTemplate('macros.txt', tmpDir)
    copyTemplate('parent.txt', tmpDir)
    copyTemplate('items.txt', tmpDir)
    copyTemplate('items.hjson', tmpDir)
    print("cd " + tmpDir)
    print("Examples of 'test/generate files':")
    print("$ gf_generate base.txt data.hjson")
    print("$ gf_generate base.txt data.hjson base.out")
    print("$ gf_generate child.txt data.hjson")
    print('$ gf_generate items.txt "class://items.hjson?type=genfiles.templates.item.Item&list=true" items.out')
  else:
    print("options: base, inheritance, class, data, test.")
    print("- base: Show a template to learn the main stuffs.")
    print("- inheritance: Show some templates to learn the inheritance.")
    print("- class: Show an example of class object.")
    print("- data: Show an example of data file.")
    print("- test: Download templates to use in learning.")
    print("")
    print("Examples of 'generate files':")
    print("$ gf_generate template.ext1")
    print("$ gf_generate template.ext1 data.ext2")
    print("$ gf_generate template.ext1 data.ext2 output.ext3")
    print('$ gf_generate items.txt "class://items.hjson?type=genfiles.templates.item.Item&list=true" items.out')

if __name__ == '__main__':
    main()
